@@ -2,63 +2,36 @@ package Mango;
 use Mojo::Base 'Mojo::EventEmitter';
 
 use Carp 'croak';
-use Mango::BSON qw(bson_doc bson_false bson_true);
+use Hash::Util::FieldHash;
+use Mango::BSON 'bson_doc';
 use Mango::Database;
 use Mango::Protocol;
 use Mojo::IOLoop;
 use Mojo::URL;
-use Mojo::Util qw(dumper md5_sum monkey_patch);
+use Mojo::Util 'dumper';
 use Scalar::Util 'weaken';
 
 use constant DEBUG => $ENV{MANGO_DEBUG} || 0;
 use constant DEFAULT_PORT => 27017;
 
-has credentials => sub { [] };
 has default_db  => 'admin';
 has hosts       => sub { [['localhost']] };
-has ioloop      => sub { Mojo::IOLoop->new };
-has j           => 0;
+has [qw(inactivity_timeout j)] => 0;
+has ioloop => sub { Mojo::IOLoop->new };
+has max_bson_size   => 16777216;
 has max_connections => 5;
-has protocol        => sub { Mango::Protocol->new };
-has w               => 1;
-has wtimeout        => 1000;
+has [qw(max_write_batch_size wtimeout)] => 1000;
+has protocol => sub { Mango::Protocol->new };
+has w => 1;
 
-our $VERSION = '0.22';
+# Private variables are not visible in the object's dump. This
+# is good for security.
+Hash::Util::FieldHash::fieldhash my %AUTH;
 
-# Operations with reply
-for my $name (qw(get_more query)) {
-  monkey_patch __PACKAGE__, $name, sub {
-    my $self = shift;
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-    my ($next, $msg) = $self->_build($name, @_);
-    $self->_start({id => $next, safe => 1, msg => $msg, cb => $cb});
-  };
-}
-
-# Operations followed by getLastError
-for my $name (qw(delete insert update)) {
-  monkey_patch __PACKAGE__, $name, sub {
-    my ($self, $namespace) = (shift, shift);
-    my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-
-    # Make sure both operations can be written together
-    my ($next, $msg) = $self->_build($name, $namespace, @_);
-    $namespace =~ s/\..+$/\.\$cmd/;
-    my $gle = bson_doc
-      getLastError => 1,
-      j            => $self->j ? bson_true : bson_false,
-      w            => $self->w,
-      wtimeout     => $self->wtimeout;
-    ($next, $gle) = $self->_build('query', $namespace, {}, 0, -1, $gle, {});
-
-    $self->_start({id => $next, safe => 1, msg => "$msg$gle", cb => $cb});
-  };
-}
+our $VERSION = '1.25';
 
 sub DESTROY { shift->_cleanup }
 
-sub new { shift->SUPER::new->from_string(@_) }
-
 sub backlog { scalar @{shift->{queue} || []} }
 
 sub db {
@@ -88,8 +61,11 @@ sub from_string {
   if (my $db = $url->path->parts->[0]) { $self->default_db($db) }
 
   # User and password
-  push @{$self->credentials}, [$self->default_db, $1, $2]
-    if ($url->userinfo // '') =~ /^([^:]+):([^:]+)$/;
+  if (($url->userinfo // '') =~ /^([^:]+):([^:]+)$/) {
+    require Mango::Auth::SCRAM;
+    $self->_auth(Mango::Auth::SCRAM->new)
+      ->_auth->_credentials([$self->default_db, $1, $2]);
+  }
 
   # Options
   my $query = $url->query;
@@ -100,31 +76,22 @@ sub from_string {
   return $self;
 }
 
-sub kill_cursors {
-  my $self = shift;
-  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-  my ($next, $msg) = $self->_build('kill_cursors', @_);
-  $self->_start({id => $next, safe => 0, msg => $msg, cb => $cb});
-}
+sub get_more { shift->_op('get_more', 1, @_) }
 
-sub _active {
-  my $self = shift;
-  return 1 if $self->backlog;
-  return !!grep { $_->{last} && !$_->{start} }
-    values %{$self->{connections} || {}};
-}
+sub kill_cursors { shift->_op('kill_cursors', 0, @_) }
+
+sub new { shift->SUPER::new->from_string(@_) }
+
+sub query { shift->_op('query', 1, @_) }
 
 sub _auth {
-  my ($self, $id, $credentials, $auth, $err, $reply) = @_;
-  my ($db, $user, $pass) = @$auth;
+  my ($self, $mode) = @_;
+  return $AUTH{$self} unless $mode;
 
-  # Run "authenticate" command with "nonce" value
-  my $nonce = $reply->{docs}[0]{nonce} // '';
-  my $key = md5_sum $nonce . $user . md5_sum "$user:mongo:$pass";
-  my $authenticate
-    = bson_doc(authenticate => 1, user => $user, nonce => $nonce, key => $key);
-  my $cb = sub { shift->_connected($id, $credentials) };
-  $self->_fast($id, $db, $authenticate, $cb);
+  $AUTH{$self} = $mode;
+  $AUTH{$self}->mango($self);
+  weaken $AUTH{$self}->{mango};
+  return $self;
 }
 
 sub _build {
@@ -137,26 +104,39 @@ sub _build {
 
 sub _cleanup {
   my $self = shift;
-  return unless my $loop = $self->_loop;
+  return unless $self->_loop(0);
 
   # Clean up connections
   delete $self->{pid};
   my $connections = delete $self->{connections};
-  $loop->remove($_) for keys %$connections;
+  for my $c (keys %$connections) {
+    my $loop = $self->_loop($connections->{$c}{nb});
+    $loop->remove($c) if $loop;
+  }
 
   # Clean up active operations
   my $queue = delete $self->{queue} || [];
-  $_->{last} and unshift @$queue, $_->{last} for values %$connections;
+  $_->{last} && !$_->{start} && unshift @$queue, $_->{last}
+    for values %$connections;
   $self->_finish(undef, $_->{cb}, 'Premature connection close') for @$queue;
 }
 
-sub _connect {
-  my ($self, $hosts) = @_;
-  my ($host, $port) = @{shift @{$hosts ||= [@{$self->hosts}]}};
+sub _close {
+  my ($self, $id) = @_;
 
+  return unless my $c = delete $self->{connections}{$id};
+  my $last = $c->{last};
+  $self->_finish(undef, $last->{cb}, 'Premature connection close') if $last;
+  $self->_connect($c->{nb}) if @{$self->{queue}};
+}
+
+sub _connect {
+  my ($self, $nb, $hosts) = @_;
+
+  my ($host, $port) = @{shift @{$hosts ||= [@{$self->hosts}]}};
   weaken $self;
   my $id;
-  $id = $self->_loop->client(
+  $id = $self->_loop($nb)->client(
     {address => $host, port => $port //= DEFAULT_PORT} => sub {
       my ($loop, $err, $stream) = @_;
 
@@ -164,54 +144,49 @@ sub _connect {
       if ($err) {
         return $self->_error($id, $err) unless @$hosts;
         delete $self->{connections}{$id};
-        return $self->_connect($hosts);
+        return $self->_connect($nb, $hosts);
       }
 
       # Connection established
-      $stream->timeout(0);
-      $stream->on(close => sub { $self->_error($id) });
+      $stream->timeout($self->inactivity_timeout);
+      $stream->on(close => sub { $self && $self->_close($id) });
       $stream->on(error => sub { $self && $self->_error($id, pop) });
       $stream->on(read => sub { $self->_read($id, pop) });
-      $self->emit(connection => $id)->_connected($id, [@{$self->credentials}]);
+
+      # Check node information with "isMaster" command
+      my $cb = sub { shift->_master($id, $nb, $hosts, pop) };
+      $self->_fast($id, $self->default_db, {isMaster => 1}, $cb);
     }
   );
-  $self->{connections}{$id} = {start => 1};
+  $self->{connections}{$id} = { nb => $nb, start => 1 };
 
   my $num = scalar keys %{$self->{connections}};
   warn "-- New connection ($host:$port:$num)\n" if DEBUG;
 }
 
-sub _connected {
-  my ($self, $id, $credentials) = @_;
-
-  # No authentication
-  return $self->_next unless my $auth = shift @$credentials;
-
-  # Run "getnonce" command followed by "authenticate"
-  my $cb = sub { shift->_auth($id, $credentials, $auth, @_) };
-  $self->_fast($id, $auth->[0], {getnonce => 1}, $cb);
-}
-
 sub _error {
   my ($self, $id, $err) = @_;
 
-  my $c    = delete $self->{connections}{$id};
-  my $last = $c->{last};
-  $last //= shift @{$self->{queue}} if $err;
-  $self->_connect if @{$self->{queue}};
-  return $err ? $self->emit(error => $err) : $self unless $last;
-  $self->_finish(undef, $last->{cb}, $err || 'Premature connection close');
+  return unless my $c = delete $self->{connections}{$id};
+  $self->_loop($c->{nb})->remove($id);
+
+  my $last = $c->{last} // shift @{$self->{queue}};
+  $self->_finish(undef, $last->{cb}, $err) if $last;
 }
 
 sub _fast {
   my ($self, $id, $db, $command, $cb) = @_;
 
   # Handle errors
-  my $protocol = $self->protocol;
-  my $wrapper  = sub {
+  my $wrapper = sub {
     my ($self, $err, $reply) = @_;
-    $err ||= $protocol->command_error($reply);
-    return $err ? $self->_error($id, $err) : $self->$cb($err, $reply);
+
+    my $doc = $reply->{docs}[0];
+    $err ||= $self->protocol->command_error($doc);
+    return $self->$cb(undef, $doc) unless $err;
+
+    return unless my $last = shift @{$self->{queue}};
+    $self->_finish(undef, $last->{cb}, $err);
   };
 
   # Skip the queue and run command right away
@@ -229,18 +204,55 @@ sub _finish {
 
 sub _id { $_[0]{id} = $_[0]->protocol->next_id($_[0]{id} // 0) }
 
-sub _loop { $_[0]{nb} ? Mojo::IOLoop->singleton : $_[0]->ioloop }
+sub _loop { $_[1] ? Mojo::IOLoop->singleton : $_[0]->ioloop }
+
+sub _master {
+  my ($self, $id, $nb, $hosts, $doc) = @_;
+
+  # Check version
+  return $self->_error($id, 'MongoDB version 3.0 required')
+    unless ($doc->{maxWireVersion} || 0) >= 3;
+
+  # Continue with authentication if we are connected to the primary
+  if ($doc->{ismaster}) {
+    return $self->_auth
+      ? $self->_auth->_authenticate($id)
+      : $self->emit(connection => $id)->_next;
+  }
+
+  # Get primary and try to connect again
+  unshift @$hosts, [$1, $2] if ($doc->{primary} // '') =~ /^(.+):(\d+)$/;
+  return $self->_error($id, "Couldn't find primary node") unless @$hosts;
+  delete $self->{connections}{$id};
+  $self->_loop($nb)->remove($id);
+  $self->_connect($nb, $hosts);
+}
 
 sub _next {
   my ($self, $op) = @_;
 
+  # Make sure all connections are saturated
   push @{$self->{queue} ||= []}, $op if $op;
-
-  my @ids = keys %{$self->{connections}};
+  my $connections = $self->{connections};
   my $start;
-  $self->_write($_) and $start++ for @ids;
-  $self->_connect
-    if $op && !$start && @{$self->{queue}} && @ids < $self->max_connections;
+  $self->_write($_) and $start++ for my @ids = keys %$connections;
+
+  # Check if we need a blocking connection
+  return unless $op;
+  return $self->_connect(0)
+    if !$op->{nb} && !grep { !$connections->{$_}{nb} } @ids;
+
+  # Check if we need more non-blocking connections
+  $self->_connect(1)
+    if !$start && @{$self->{queue}} && @ids < $self->max_connections;
+}
+
+sub _op {
+  my ($self, $op, $safe) = (shift, shift, shift);
+  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
+  my ($next, $msg) = $self->_build($op, @_);
+  $self->_start(
+    {id => $next, safe => $safe, msg => $msg, nb => !!$cb, cb => $cb});
 }
 
 sub _read {
@@ -263,39 +275,14 @@ sub _start {
   $self->_cleanup unless ($self->{pid} //= $$) eq $$;
 
   # Non-blocking
-  if ($op->{cb}) {
-
-    # Start non-blocking
-    unless ($self->{nb}) {
-      croak 'Blocking operation in progress' if $self->_active;
-      warn "-- Switching to non-blocking mode\n" if DEBUG;
-      $self->_cleanup;
-      $self->{nb}++;
-    }
-
-    return $self->_next($op);
-  }
-
-  # Start blocking
-  if ($self->{nb}) {
-    croak 'Non-blocking operations in progress' if $self->_active;
-    warn "-- Switching to blocking mode\n" if DEBUG;
-    $self->_cleanup;
-    delete $self->{nb};
-  }
+  return $self->_next($op) if $op->{cb};
 
+  # Blocking
   my ($err, $reply);
-  $op->{cb} = sub {
-    (my $self, $err, $reply) = @_;
-    $self->ioloop->stop;
-  };
+  $op->{cb} = sub { shift->ioloop->stop; ($err, $reply) = @_ };
   $self->_next($op);
   $self->ioloop->start;
-
-  # Throw blocking errors
-  croak $err if $err;
-
-  return $reply;
+  return $err ? croak $err : $reply;
 }
 
 sub _write {
@@ -304,15 +291,22 @@ sub _write {
   # Make sure connection has not been corrupted while event loop was stopped
   my $c = $self->{connections}{$id};
   return $c->{start} if $c->{last};
-  my $loop = $self->_loop;
+  my $loop = $self->_loop($c->{nb});
   return undef unless my $stream = $loop->stream($id);
   if (!$loop->is_running && $stream->is_readable) {
     $stream->close;
     return undef;
   }
 
+  # Fast operation
   delete $c->{start} unless my $last = delete $c->{fast};
-  return $c->{start} unless $c->{last} = $last ||= shift @{$self->{queue}};
+
+  # Blocking operations have a higher precedence
+  return $c->{start}
+    unless $last || ($c->{nb} xor !($self->{queue}->[-1] || {})->{nb});
+  $last ||= $c->{nb} ? shift @{$self->{queue}} : pop @{$self->{queue}};
+
+  return $c->{start} unless $c->{last} = $last;
   warn "-- Client >>> Server (#$last->{id})\n" if DEBUG;
   $stream->write(delete $last->{msg});
 
@@ -355,18 +349,7 @@ Mango - Pure-Perl non-blocking I/O MongoDB driver
   my $oid = $mango->db('test')->collection('foo')
     ->insert({data => bson_bin("\x00\x01"), now => bson_time});
 
-  # Blocking parallel find (does not work inside a running event loop)
-  my $delay = Mojo::IOLoop->delay;
-  for my $name (qw(sri marty)) {
-    my $end = $delay->begin(0);
-    $mango->db('test')->collection('users')->find({name => $name})->all(sub {
-      my ($cursor, $err, $docs) = @_;
-      $end->(@$docs);
-    });
-  }
-  my @docs = $delay->wait;
-
-  # Non-blocking parallel find (does work inside a running event loop)
+  # Non-blocking concurrent find
   my $delay = Mojo::IOLoop->delay(sub {
     my ($delay, @docs) = @_;
     ...
@@ -378,7 +361,7 @@ Mango - Pure-Perl non-blocking I/O MongoDB driver
       $end->(@$docs);
     });
   }
-  $delay->wait unless Mojo::IOLoop->is_running;
+  $delay->wait;
 
   # Event loops such as AnyEvent are supported through EV
   use EV;
@@ -397,17 +380,13 @@ with the L<Mojolicious> real-time web framework, and with multiple event loop
 support. Since MongoDB is still changing rapidly, only the latest stable
 version is supported.
 
+For MongoDB 2.6 support, use L<Mango> 1.16.
+
 To learn more about MongoDB you should take a look at the
 L<official documentation|http://docs.mongodb.org>, the documentation included
 in this distribution is no replacement for it.
 
-Note that this whole distribution is EXPERIMENTAL and will change without
-warning!
-
-Most of the API is not changing much anymore, but you should wait for a stable
-1.0 release before using any of the modules in this distribution in a
-production environment. Unsafe operations are not supported, so far this is
-considered a feature.
+Look at L<Mango::Collection> for CRUD operations.
 
 Many arguments passed to methods as well as values of attributes get
 serialized to BSON with L<Mango::BSON>, which provides many helper functions
@@ -415,11 +394,12 @@ you can use to generate data types that are not available natively in Perl.
 All connections will be reset automatically if a new process has been forked,
 this allows multiple processes to share the same L<Mango> object safely.
 
-For better scalability (epoll, kqueue) and to provide IPv6 as well as TLS
-support, the optional modules L<EV> (4.0+), L<IO::Socket::IP> (0.16+) and
-L<IO::Socket::SSL> (1.75+) will be used automatically by L<Mojo::IOLoop> if
-they are installed. Individual features can also be disabled with the
-MOJO_NO_IPV6 and MOJO_NO_TLS environment variables.
+For better scalability (epoll, kqueue) and to provide IPv6, SOCKS5 as well as
+TLS support, the optional modules L<EV> (4.0+), L<IO::Socket::IP> (0.20+),
+L<IO::Socket::Socks> (0.64+) and L<IO::Socket::SSL> (1.84+) will be used
+automatically if they are installed. Individual features can also be disabled
+with the C<MOJO_NO_IPV6>, C<MOJO_NO_SOCKS> and C<MOJO_NO_TLS> environment
+variables.
 
 =head1 EVENTS
 
@@ -435,31 +415,10 @@ following new ones.
 
 Emitted when a new connection has been established.
 
-=head2 error
-
-  $mango->on(error => sub {
-    my ($mango, $err) = @_;
-    ...
-  });
-
-Emitted if an error occurs that can't be associated with an operation.
-
-  $mango->on(error => sub {
-    my ($mango, $err) = @_;
-    say "This looks bad: $err";
-  });
-
 =head1 ATTRIBUTES
 
 L<Mango> implements the following attributes.
 
-=head2 credentials
-
-  my $credentials = $mango->credentials;
-  $mango          = $mango->credentials([['test', 'sri', 's3cret']]);
-
-Authentication credentials that will be used on every reconnect.
-
 =head2 default_db
 
   my $name = $mango->default_db;
@@ -474,6 +433,15 @@ Default database, defaults to C<admin>.
 
 Servers to connect to, defaults to C<localhost> and port C<27017>.
 
+=head2 inactivity_timeout
+
+  my $timeout = $mango->inactivity_timeout;
+  $mango      = $mango->inactivity_timeout(15);
+
+Maximum amount of time in seconds a connection can be inactive before getting
+closed, defaults to C<0>. Setting the value to C<0> will allow connections to
+be inactive indefinitely.
+
 =head2 ioloop
 
   my $loop = $mango->ioloop;
@@ -489,6 +457,13 @@ L<Mojo::IOLoop> object.
 
 Wait for all operations to have reached the journal, defaults to C<0>.
 
+=head2 max_bson_size
+
+  my $max = $mango->max_bson_size;
+  $mango  = $mango->max_bson_size(16777216);
+
+Maximum size for BSON documents in bytes, defaults to C<16777216> (16MB).
+
 =head2 max_connections
 
   my $max = $mango->max_connections;
@@ -497,6 +472,13 @@ Wait for all operations to have reached the journal, defaults to C<0>.
 Maximum number of connections to use for non-blocking operations, defaults to
 C<5>.
 
+=head2 max_write_batch_size
+
+  my $max = $mango->max_write_batch_size;
+  $mango  = $mango->max_write_batch_size(1000);
+
+ Maximum number of write operations to batch together, defaults to C<1000>.
+
 =head2 protocol
 
   my $protocol = $mango->protocol;
@@ -525,14 +507,6 @@ Timeout for write propagation in milliseconds, defaults to C<1000>.
 L<Mango> inherits all methods from L<Mojo::Base> and implements the following
 new ones.
 
-=head2 new
-
-  my $mango = Mango->new;
-  my $mango = Mango->new('mongodb://sri:s3cret@localhost:3000/test?w=2');
-
-Construct a new L<Mango> object and parse connection string with
-L</"from_string"> if necessary.
-
 =head2 backlog
 
   my $num = $mango->backlog;
@@ -544,23 +518,10 @@ Number of queued operations that have not yet been assigned to a connection.
   my $db = $mango->db;
   my $db = $mango->db('test');
 
-Get L<Mango::Database> object for database, uses L</"default_db"> if no name
+Build L<Mango::Database> object for database, uses L</"default_db"> if no name
 is provided. Note that the reference L<Mango::Database/"mango"> is weakened,
 so the L<Mango> object needs to be referenced elsewhere as well.
 
-=head2 delete
-
-  my $reply = $mango->delete($namespace, $flags, $query);
-
-Perform low level C<delete> operation followed by C<getLastError> command. You
-can also append a callback to perform operation non-blocking.
-
-  $mango->delete(($namespace, $flags, $query) => sub {
-    my ($mango, $err, $reply) = @_;
-    ...
-  });
-  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
-
 =head2 from_string
 
   $mango
@@ -572,7 +533,7 @@ Parse configuration from connection string.
 
   my $reply = $mango->get_more($namespace, $return, $cursor);
 
-Perform low level C<get_more> operation. You can also append a callback to
+Perform low level C<GET_MORE> operation. You can also append a callback to
 perform operation non-blocking.
 
   $mango->get_more(($namespace, $return, $cursor) => sub {
@@ -581,24 +542,11 @@ perform operation non-blocking.
   });
   Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
 
-=head2 insert
-
-  my $reply = $mango->insert($namespace, $flags, @docs);
-
-Perform low level C<insert> operation followed by C<getLastError> command. You
-can also append a callback to perform operation non-blocking.
-
-  $mango->insert(($namespace, $flags, @docs) => sub {
-    my ($mango, $err, $reply) = @_;
-    ...
-  });
-  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
-
 =head2 kill_cursors
 
   $mango->kill_cursors(@ids);
 
-Perform low level C<kill_cursors> operation. You can also append a callback to
+Perform low level C<KILL_CURSORS> operation. You can also append a callback to
 perform operation non-blocking.
 
     $mango->kill_cursors(@ids => sub {
@@ -607,12 +555,24 @@ perform operation non-blocking.
     });
     Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
 
+=head2 new
+
+  my $mango = Mango->new;
+  my $mango = Mango->new('mongodb://sri:s3cret@localhost:3000/test?w=2');
+
+Construct a new L<Mango> object and parse connection string with
+L</"from_string"> if necessary.
+
+If a username and password are provided, Mango will try to authenticate using
+SCRAM-SHA1. B<Warning:> this will require L<Authen::SCRAM> which is not
+installed by default.
+
 =head2 query
 
   my $reply
     = $mango->query($namespace, $flags, $skip, $return, $query, $fields);
 
-Perform low level C<query> operation. You can also append a callback to
+Perform low level C<QUERY> operation. You can also append a callback to
 perform operation non-blocking.
 
   $mango->query(($namespace, $flags, $skip, $return, $query, $fields) => sub {
@@ -621,19 +581,6 @@ perform operation non-blocking.
   });
   Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
 
-=head2 update
-
-  my $reply = $mango->update($namespace, $flags, $query, $update);
-
-Perform low level C<update> operation followed by C<getLastError> command. You
-can also append a callback to perform operation non-blocking.
-
-  $mango->update(($namespace, $flags, $query, $update) => sub {
-    my ($mango, $err, $reply) = @_;
-    ...
-  });
-  Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
-
 =head1 DEBUGGING
 
 You can set the C<MANGO_DEBUG> environment variable to get some advanced
@@ -650,25 +597,32 @@ L<Drip Depot|http://www.dripdepot.com>, thank you!
 
 Sebastian Riedel, C<sri@cpan.org>.
 
+Current maintainer: Olivier Duclos C<odc@cpan.org>.
+
 =head1 CREDITS
 
 In alphabetical order:
 
 =over 2
 
+alexbyk
+
 Andrey Khozov
 
+Colin Cyr
+
 =back
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2013, Sebastian Riedel.
+Copyright (C) 2013-2014, Sebastian Riedel.
 
 This program is free software, you can redistribute it and/or modify it under
 the terms of the Artistic License version 2.0.
 
 =head1 SEE ALSO
 
-L<Mojolicious::Guides>, L<http://mojolicio.us>.
+L<https://github.com/oliwer/mango>, L<Mojolicious::Guides>,
+L<http://mojolicio.us>.
 
 =cut
